import adani as ad
import oldadani as old
import numpy as np

def test_as1_oldadani():
    for kind in ['2', 'L']:
        hs = ad.HighScaleCoefficientFunction(1, kind, 'g')
        for x in np.geomspace(1e-5, 1., 100, endpoint=False):
            for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
                res1 = hs.fx(x, m2Q2, 0., 0)
                res2 = old.C2_g1_highscale(x, m2Q2) if kind == '2' else old.CL_g1_highscale(x)
                np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_as2_oldadani():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            hs = ad.HighScaleCoefficientFunction(2, kind, channel)
            for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
                    for m2mu2 in np.geomspace(1e-4, 1e-2, 10):
                        res1 = hs.fx(x, m2Q2, m2mu2, 0)
                        if (kind == '2' and channel == 'g'):
                            res2 = old.C2_g2_highscale(x, m2Q2, m2mu2)
                        if (kind == '2' and channel == 'q'):
                            res2 = old.C2_ps2_highscale(x, m2Q2, m2mu2)
                        if (kind == 'L' and channel == 'g'):
                            res2 = old.CL_g2_highscale(x, m2Q2, m2mu2)
                        if (kind == 'L' and channel == 'q'):
                            res2 = old.CL_ps2_highscale(x, m2Q2, m2mu2)
                        np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_as3_oldadani():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            version = "exact" if channel == 'q' else "abmp"
            hs = ad.HighScaleCoefficientFunction(3, kind, channel, version)
            for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
                    for m2mu2 in np.geomspace(1e-4, 1e-2, 10):
                        for nf in range(6 + 1):
                            res1 = hs.fx(x, m2Q2, m2mu2, nf)
                            if (kind == '2' and channel == 'g'):
                                res2 = old.C2_g3_highscale(x, m2Q2, m2mu2, nf)
                            if (kind == '2' and channel == 'q'):
                                res2 = old.C2_ps3_highscale(x, m2Q2, m2mu2, nf)
                            if (kind == 'L' and channel == 'g'):
                                res2 = old.CL_g3_highscale(x, m2Q2, m2mu2, nf)
                            if (kind == 'L' and channel == 'q'):
                                res2 = old.CL_ps3_highscale(x, m2Q2, m2mu2, nf)
                            np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_splitlogs_vs_highscale():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            for hs_version in ["exact", "abmp", "klmv"]:
                if channel == 'q' and hs_version == "abmp":
                    continue
                if channel == 'g' and hs_version == "exact":
                    continue
                hs = ad.HighScaleCoefficientFunction(3, kind, channel, hs_version)
                hs_split = ad.HighScaleSplitLogs(3, kind, channel, hs_version)
                for x in np.geomspace(1e-5, 1, 100, endpoint=False):
                    for xi in np.geomspace(1e-2, 1e4, 10, endpoint=True):
                        for nf in range(1, 6 + 1):
                            res1 = hs.fxBand(x, 1./xi, 1./xi, nf)
                            res2 = hs_split.fxBand(x, 1./xi, nf)
                            vec1 = [res1.GetCentral(), res1.GetHigher(), res1.GetLower()]
                            vec2 = [res2.GetCentral(), res2.GetHigher(), res2.GetLower()]
                            np.testing.assert_allclose(vec1, vec2, rtol = 1e-4)
                            central1 = hs.fx(x, 1./xi, 1./xi, nf)
                            central2 = hs_split.fx(x, 1./xi, nf)
                            np.testing.assert_allclose(central1, vec1[0], rtol = 1e-4)
                            np.testing.assert_allclose(central2, vec2[0], rtol = 1e-4)

def test_split_logs():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            for hs_version in ["exact", "abmp", "klmv"]:
                if channel == 'q' and hs_version == "abmp":
                    continue
                if channel == 'g' and hs_version == "exact":
                    continue
                hs_split = ad.HighScaleSplitLogs(3, kind, channel, hs_version)
                for x in np.geomspace(1e-5, 1, 100, endpoint=False):
                    for xi in np.geomspace(1e-2, 1e4, 10, endpoint=True):
                        for nf in range(1, 6 + 1):
                            logQ = np.log(1/xi)
                            res1 = (
                                hs_split.N3LL(x, nf)
                                + hs_split.N2LL(x, nf) * logQ
                                + hs_split.NLL(x, nf) * logQ**2
                                + hs_split.LL(x, nf) * logQ**3
                            )
                            res2 = hs_split.fxBand(x, 1./xi, nf)
                            vec1 = [res1.GetCentral(), res1.GetHigher(), res1.GetLower()]
                            vec2 = [res2.GetCentral(), res2.GetHigher(), res2.GetLower()]
                            np.testing.assert_allclose(vec1, vec2, rtol = 1e-7)
                            central1 = (
                                hs_split.LL(x, nf) * logQ**3
                                + hs_split.NLL(x, nf) * logQ**2
                                + hs_split.N2LL(x, nf) * logQ
                                + hs_split.N3LL(x, nf).GetCentral()
                            )
                            central2 = hs_split.fx(x, 1./xi, nf)
                            np.testing.assert_allclose(central1, vec1[0], rtol = 1e-7)
                            np.testing.assert_allclose(central2, vec2[0], rtol = 1e-7)



import adani as ad
import oldadani as old
import numpy as np

def test_nll():
    for nll in [True, False]:
        he = ad.HighEnergyCoefficientFunction(3, "2", "g", NLL=nll)
        hehs = ad.HighEnergyHighScaleCoefficientFunction(3, "2", "g", NLL=nll)
        pt = ad.PowerTermsCoefficientFunction(3, "2", "g", NLL=nll)
        assert he.GetNLL() == nll
        assert hehs.GetNLL() == nll
        assert pt.GetNLL() == nll

def test_highenergy_as2_oldadani():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            he = ad.HighEnergyCoefficientFunction(2, kind, channel)
            for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
                    for m2mu2 in np.geomspace(1e-4, 1e-2, 10):
                        res1 = he.fx(x, m2Q2, m2mu2, 0)
                        if (kind == '2' and channel == 'g'):
                            res2 = old.C2_g2_highenergy(x, m2Q2, m2mu2)
                        if (kind == '2' and channel == 'q'):
                            res2 = old.C2_ps2_highenergy(x, m2Q2, m2mu2)
                        if (kind == 'L' and channel == 'g'):
                            res2 = old.CL_g2_highenergy(x, m2Q2, m2mu2)
                        if (kind == 'L' and channel == 'q'):
                            res2 = old.CL_ps2_highenergy(x, m2Q2, m2mu2)
                        np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_highenergy_as3_oldadani():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            he = ad.HighEnergyCoefficientFunction(3, kind, channel)
            for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
                    for m2mu2 in np.geomspace(1e-4, 1e-2, 10):
                        for nf in range(6 + 1):
                            res1 = he.fx(x, m2Q2, m2mu2, nf)
                            if (kind == '2' and channel == 'g'):
                                res2 = old.C2_g3_highenergy(x, m2Q2, m2mu2, nf, 0)
                            if (kind == '2' and channel == 'q'):
                                res2 = old.C2_ps3_highenergy(x, m2Q2, m2mu2, nf, 0)
                            if (kind == 'L' and channel == 'g'):
                                res2 = old.CL_g3_highenergy(x, m2Q2, m2mu2, nf, 0)
                            if (kind == 'L' and channel == 'q'):
                                res2 = old.CL_ps3_highenergy(x, m2Q2, m2mu2, nf, 0)
                            np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_highenergyhighscale_as2_oldadani():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            he = ad.HighEnergyHighScaleCoefficientFunction(2, kind, channel)
            for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
                    for m2mu2 in np.geomspace(1e-4, 1e-2, 10):
                        res1 = he.fx(x, m2Q2, m2mu2, 0)
                        if (kind == '2' and channel == 'g'):
                            res2 = old.C2_g2_highenergy_highscale(x, m2Q2, m2mu2)
                        if (kind == '2' and channel == 'q'):
                            res2 = old.C2_ps2_highenergy_highscale(x, m2Q2, m2mu2)
                        if (kind == 'L' and channel == 'g'):
                            res2 = old.CL_g2_highenergy_highscale(x, m2Q2, m2mu2)
                        if (kind == 'L' and channel == 'q'):
                            res2 = old.CL_ps2_highenergy_highscale(x, m2Q2, m2mu2)
                        np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_highenergyhighscale_as3_oldadani():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            he = ad.HighEnergyHighScaleCoefficientFunction(3, kind, channel)
            for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
                    for m2mu2 in np.geomspace(1e-4, 1e-2, 10):
                        for nf in range(6 + 1):
                            res1 = he.fx(x, m2Q2, m2mu2, nf)
                            if (kind == '2' and channel == 'g'):
                                res2 = old.C2_g3_highenergy_highscale(x, m2Q2, m2mu2, nf, 0)
                            if (kind == '2' and channel == 'q'):
                                res2 = old.C2_ps3_highenergy_highscale(x, m2Q2, m2mu2, nf, 0)
                            if (kind == 'L' and channel == 'g'):
                                res2 = old.CL_g3_highenergy_highscale(x, m2Q2, m2mu2, nf, 0)
                            if (kind == 'L' and channel == 'q'):
                                res2 = old.CL_ps3_highenergy_highscale(x, m2Q2, m2mu2, nf, 0)
                            np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_powerterms_as2_oldadani():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            he = ad.PowerTermsCoefficientFunction(2, kind, channel)
            for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
                    for m2mu2 in np.geomspace(1e-4, 1e-2, 10):
                        res1 = he.fx(x, m2Q2, m2mu2, 0)
                        if (kind == '2' and channel == 'g'):
                            res2 = old.C2_g2_power_terms(x, m2Q2, m2mu2)
                        if (kind == '2' and channel == 'q'):
                            res2 = old.C2_ps2_power_terms(x, m2Q2, m2mu2)
                        if (kind == 'L' and channel == 'g'):
                            res2 = old.CL_g2_power_terms(x, m2Q2, m2mu2)
                        if (kind == 'L' and channel == 'q'):
                            res2 = old.CL_ps2_power_terms(x, m2Q2, m2mu2)
                        np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_powerterms_as3_oldadani():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            he = ad.PowerTermsCoefficientFunction(3, kind, channel)
            for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
                    for m2mu2 in np.geomspace(1e-4, 1e-2, 10):
                        for nf in range(6 + 1):
                            res1 = he.fx(x, m2Q2, m2mu2, nf)
                            if (kind == '2' and channel == 'g'):
                                res2 = old.C2_g3_power_terms(x, m2Q2, m2mu2, nf, 0)
                            if (kind == '2' and channel == 'q'):
                                res2 = old.C2_ps3_power_terms(x, m2Q2, m2mu2, nf, 0)
                            if (kind == 'L' and channel == 'g'):
                                res2 = old.CL_g3_power_terms(x, m2Q2, m2mu2, nf, 0)
                            if (kind == 'L' and channel == 'q'):
                                res2 = old.CL_ps3_power_terms(x, m2Q2, m2mu2, nf, 0)
                            np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_powerterms():
    for order in range(1, 3 + 1):
        for kind in ['2', 'L']:
            for channel in ['g', 'q']:
                if channel == 'q' and order == 1:
                    continue
                for NLL in [True, False]:
                    he = ad.HighEnergyCoefficientFunction(order, kind, channel, NLL)
                    hehs = ad.HighEnergyHighScaleCoefficientFunction(order, kind, channel, NLL)
                    pt = ad.PowerTermsCoefficientFunction(order, kind, channel, NLL)
                    for nf in range(1, 6 + 1):
                        for m2mu2 in np.geomspace(1e-4, 1e2, 10):
                            for m2Q2 in np.geomspace(1e-4, 1e2, 10):
                                for x in np.geomspace(1e-5, 1., 10, endpoint=False):
                                    res1 = pt.fxBand(x, m2Q2, m2mu2, nf)

                                    res2_c = he.fxBand(x, m2Q2, m2mu2, nf).GetCentral() - hehs.fxBand(x, m2Q2, m2mu2, nf).GetCentral()
                                    tmp1 = he.fxBand(x, m2Q2, m2mu2, nf).GetHigher() - hehs.fxBand(x, m2Q2, m2mu2, nf).GetHigher()
                                    tmp2 = he.fxBand(x, m2Q2, m2mu2, nf).GetLower() - hehs.fxBand(x, m2Q2, m2mu2, nf).GetLower()
                                    if tmp1 > tmp2:
                                        res2_h = tmp1
                                        res2_l = tmp2
                                    else:
                                        res2_h = tmp2
                                        res2_l = tmp1

                                    np.testing.assert_allclose(res1.GetCentral(), res2_c, rtol=1e-7)
                                    np.testing.assert_allclose(res1.GetHigher(), res2_h, rtol=1e-7)
                                    np.testing.assert_allclose(res1.GetLower(), res2_l, rtol=1e-7)
                    del he
                    del pt
                    del hehs



import adani as ad
import oldadani as old
import numpy as np

def test_as2():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            asy = ad.AsymptoticCoefficientFunction(2, kind, channel)
            for m2Q2 in np.geomspace(1e-2, 1e4, 10):
                for m2mu2 in np.geomspace(1e-2, 1e4, 10):
                    for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                        res1 = asy.fx(x, m2Q2, m2mu2, 1)
                        if kind == '2' and channel == 'g':
                            res2 = old.C2_g2_asymptotic(x, m2Q2, m2mu2)
                        if kind == '2' and channel == 'q':
                            res2 = old.C2_ps2_asymptotic(x, m2Q2, m2mu2)
                        if kind == 'L' and channel == 'g':
                            res2 = old.CL_g2_asymptotic(x, m2Q2, m2mu2)
                        if kind == 'L' and channel == 'q':
                            res2 = old.CL_ps2_asymptotic(x, m2Q2, m2mu2)
                        np.testing.assert_allclose(res1, res2, rtol=1e-7)


def test_as3():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            hs_version = "exact" if channel == 'q' else "abmp"
            asy = ad.AsymptoticCoefficientFunction(3, kind, channel, True, hs_version)
            for m2Q2 in np.geomspace(1e-2, 1e4, 10):
                for m2mu2 in np.geomspace(1e-2, 1e4, 10):
                    for nf in range(1, 6+1):
                        for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                            res1 = asy.fx(x, m2Q2, m2mu2, nf)
                            if kind == '2' and channel == 'g':
                                res2 = old.C2_g3_asymptotic(x, m2Q2, m2mu2, nf)
                            if kind == '2' and channel == 'q':
                                res2 = old.C2_ps3_asymptotic(x, m2Q2, m2mu2, nf)
                            if kind == 'L' and channel == 'g':
                                res2 = old.CL_g3_asymptotic(x, m2Q2, m2mu2, nf)
                            if kind == 'L' and channel == 'q':
                                res2 = old.CL_ps3_asymptotic(x, m2Q2, m2mu2, nf)
                            np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_asy():
    for order in range(1, 3 + 1):
        for kind in ['2', 'L']:
            for channel in ['g', 'q']:
                if order == 1 and channel == 'q':
                    continue
                for hs_version in ["exact", "abmp", "klmv"]:
                    if channel == 'q' and hs_version == "abmp":
                        continue
                    if channel == 'g' and hs_version == "exact":
                        continue
                    hs = ad.HighScaleCoefficientFunction(order, kind, channel, hs_version)
                    for nll in [True, False]:
                        pt = ad.PowerTermsCoefficientFunction(order, kind, channel, nll)
                        asy = ad.AsymptoticCoefficientFunction(order, kind, channel, nll, hs_version)
                        for nf in range(1, 6 + 1):
                            for m2mu2 in np.geomspace(1e-4, 1e2, 10):
                                for m2Q2 in np.geomspace(1e-4, 1e2, 10):
                                    for x in np.geomspace(1e-5, 1., 10, endpoint=False):
                                        res1 = pt.fxBand(x, m2Q2, m2mu2, nf) + hs.fxBand(x, m2Q2, m2mu2, nf)
                                        res2 = asy.fxBand(x, m2Q2, m2mu2, nf)

                                        res1_c = pt.fx(x, m2Q2, m2mu2, nf) + hs.fx(x, m2Q2, m2mu2, nf)
                                        res2_c = asy.fx(x, m2Q2, m2mu2, nf)

                                        np.testing.assert_allclose(res1.GetCentral(), res2.GetCentral(), rtol=1e-7)
                                        np.testing.assert_allclose(res1.GetHigher(), res2.GetHigher(), rtol=1e-7)
                                        np.testing.assert_allclose(res1.GetLower(), res2.GetLower(), rtol=1e-7)
                                        np.testing.assert_allclose(res1_c, res1.GetCentral(), rtol=1e-7)
                                        np.testing.assert_allclose(res2_c, res2.GetCentral(), rtol=1e-7)



import adani as ad
import oldadani as oldad
import numpy as np

def test_coeff_func():
    for order in range(1, 3 + 1):
        for kind in ["2", "L"]:
            for channel in ["q", "g"]:
                if order == 1 and channel == "q":
                    continue
                app = ad.ApproximateCoefficientFunction(order, kind, channel)
                assert app.GetOrder() == order
                assert app.GetKind() == kind
                assert app.GetChannel() == channel
                del app

def test_mudependent_terms():
    for order in [2, 3]:
        for channel in ['g', 'q']:
            for kind in ['2', 'L']:
                for dim in [1000]:
                    for MCcalls in [20000]:
                        for mf in ["analytical", "double_numerical", "monte_carlo"]:
                            for abserr in [1e-3]:
                                relerr = abserr
                                massive = ad.ExactCoefficientFunction(order, kind, channel, abserr, relerr, dim, mf, MCcalls)
                                app = ad.ApproximateCoefficientFunction(order, kind, channel, True, "klmv", abserr, relerr, dim, mf, MCcalls)
                                x = np.geomspace(1e-5, 1., 5, endpoint=True)
                                for xi in np.geomspace(1e-2, 1e4, 4, endpoint=True):
                                    for nf in [4, 5]:
                                        res1 = [massive.MuDependentTerms(x_, 1/xi, 1/xi, nf) for x_ in x]
                                        res2 = [app.MuDependentTerms(x_, 1/xi, 1/xi, nf) for x_ in x]
                                        np.testing.assert_allclose(res1, res2, rtol=1e-7)
                                del massive
                                del app

def test_as2_muindep_oldversion():
    for channel in ['g', 'q']:
        for kind in ['2', 'L']:
            app = ad.ApproximateCoefficientFunction(2, kind, channel)
            for xi in np.geomspace(1e-2, 1e2, 10):
                m2Q2 = 1/xi
                for x in np.geomspace(1e-5, 1, 10):
                    for v in [0, 1, -1]:
                        if kind == '2':
                            if channel == 'g':
                                res_old = oldad.C2_g2_approximation(x, m2Q2, 1., v)
                            if channel == 'q':
                                res_old = oldad.C2_ps2_approximation(x, m2Q2, 1., v)
                        if kind == 'L':
                            if channel == 'g':
                                res_old = oldad.CL_g2_approximation(x, m2Q2, 1., v)
                            if channel == 'q':
                                res_old = oldad.CL_ps2_approximation(x, m2Q2, 1., v)
                        tmp = app.MuIndependentTermsBand(x, m2Q2, 1)
                        if v == 0:
                            res_new = tmp.GetCentral()
                        if v == -1:
                            res_new = tmp.GetLower()
                        if v == 1:
                            res_new = tmp.GetHigher()
                        np.testing.assert_allclose(res_old, res_new, rtol=1e-7)

def test_as3_muindep_oldversion():
    for channel in ['g', 'q']:
        for kind in ['2', 'L']:
            highscale_version = "exact" if channel == 'q' else "abmp"
            app = ad.ApproximateCoefficientFunction(3, kind, channel,True, highscale_version)
            for xi in np.geomspace(1e-2, 1e2, 10):
                m2Q2 = 1/xi
                for x in np.geomspace(1e-5, 1, 10):
                    for nf in range(1, 6 + 1):
                        for v in [0, 1, -1]:
                            if kind == '2':
                                if channel == 'g':
                                    res_old = oldad.C2_g3_approximation(x, m2Q2, 1., nf, v)
                                if channel == 'q':
                                    res_old = oldad.C2_ps3_approximation(x, m2Q2, 1., nf, v)
                            if kind == 'L':
                                if channel == 'g':
                                    res_old = oldad.CL_g3_approximation(x, m2Q2, 1., nf, v)
                                if channel == 'q':
                                    res_old = oldad.CL_ps3_approximation(x, m2Q2, 1., nf, v)
                            tmp = app.MuIndependentTermsBand(x, m2Q2, nf)
                            if v == 0:
                                res_new = tmp.GetCentral()
                            if v == -1:
                                res_new = tmp.GetLower()
                            if v == 1:
                                res_new = tmp.GetHigher()
                            np.testing.assert_allclose(res_old, res_new, rtol=1e-7)

def test_mudep_as2_oldversion():
    for channel in ['g', 'q']:
        for kind in ['2', 'L']:
            app = ad.ApproximateCoefficientFunction(2, kind, channel)
            for xi in np.geomspace(1e-2, 1e2, 10):
                m2Q2 = 1/xi
                xmax = 1/(1 + 4*m2Q2)
                Lmu = -np.log(m2Q2)
                for x in np.geomspace(1e-5, 1, 10):
                    for nf in [3, 4]:
                        if kind == '2':
                            if channel == 'g':
                                res_old = oldad.C2_g21(x, m2Q2) * Lmu
                            if channel == 'q':
                                res_old = oldad.C2_ps21(x, m2Q2) * Lmu
                        if kind == 'L':
                            if channel == 'g':
                                res_old = oldad.CL_g21(x, m2Q2) * Lmu
                            if channel == 'q':
                                res_old = oldad.CL_ps21(x, m2Q2) * Lmu
                        if x > xmax:
                            res_old = 0.
                        res_new = app.MuDependentTerms(x, m2Q2, m2Q2, nf)
                        np.testing.assert_allclose(res_old, res_new, rtol=1e-7)
            del app

# def test_mudep_as3_oldversion():
#     for channel in ['g', 'q']:
#         for kind in ['2', 'L']:
#             highscale_version = "exact" if channel == 'q' else "abmp"
#             for mf in [False]:
#                 app = ad.ApproximateCoefficientFunction(3, kind, channel, True, highscale_version, 1e-3, 1e-3, 1000, mf, 25000)
#                 for xi in np.geomspace(1e-2, 1e2, 10):
#                     m2Q2 = 1/xi
#                     xmax = 1/(1 + 4*m2Q2)
#                     Lmu = -np.log(m2Q2)
#                     for x in np.geomspace(1e-5, 1, 10):
#                         for nf in [3, 4]:
#                             if kind == '2':
#                                 if channel == 'g':
#                                     res_old = oldad.C2_g31(x, m2Q2, nf) * Lmu + oldad.C2_g32(x, m2Q2, nf, mf) * Lmu**2
#                                 if channel == 'q':
#                                     res_old = oldad.C2_ps31(x, m2Q2, nf) * Lmu + oldad.C2_ps32(x, m2Q2, nf) * Lmu**2
#                             if kind == 'L':
#                                 if channel == 'g':
#                                     res_old = oldad.CL_g31(x, m2Q2, nf) * Lmu + oldad.CL_g32(x, m2Q2, nf, mf) * Lmu**2
#                                 if channel == 'q':
#                                     res_old = oldad.CL_ps31(x, m2Q2, nf) * Lmu + oldad.CL_ps32(x, m2Q2, nf) * Lmu**2
#                             if x > xmax:
#                                 res_old = 0.
#                             res_new = app.MuDependentTerms(x, m2Q2, m2Q2, nf)
#                             np.testing.assert_allclose(res_old, res_new, rtol=1e-7)
                # del app

def test_klmv_as2():
    for channel in ['g', 'q']:
        app = ad.ApproximateCoefficientFunctionKLMV(2, '2', channel, "abmp")
        for xi in np.geomspace(1e-2, 1e2, 10):
            m2Q2 = 1/xi
            for x in np.geomspace(1e-5, 1, 10):
                if channel == 'g':
                    tmpA = oldad.C2_g2_approximationA_klmv(x, m2Q2, 1.)
                    tmpB = oldad.C2_g2_approximationB_klmv(x, m2Q2, 1.)
                if channel == 'q':
                    tmpA = oldad.C2_ps2_approximationA_klmv(x, m2Q2, 1.)
                    tmpB = oldad.C2_ps2_approximationB_klmv(x, m2Q2, 1.)
                if tmpA > tmpB:
                    higher = tmpA
                    lower = tmpB
                else:
                    higher = tmpB
                    lower = tmpA
                tmp = app.MuIndependentTermsBand(x, m2Q2, 1)
                np.testing.assert_allclose(higher, tmp.GetHigher(), rtol=1e-7)
                np.testing.assert_allclose(lower, tmp.GetLower(), rtol=1e-7)

def test_klmv_as3():
    for channel in ['g', 'q']:
        hs_version = "abmp" if channel == 'g' else 'exact'
        app = ad.ApproximateCoefficientFunctionKLMV(3, '2', channel, hs_version)
        for xi in np.geomspace(1e-2, 1e2, 10):
            m2Q2 = 1/xi
            for x in np.geomspace(1e-5, 1, 10):
                for nf in [4, 5, 6]:
                    if channel == 'g':
                        tmpA = oldad.C2_g3_approximationA_klmv(x, m2Q2, 1., nf, 0)
                        tmpB = oldad.C2_g3_approximationB_klmv(x, m2Q2, 1., nf, 0)
                    if channel == 'q':
                        tmpA = oldad.C2_ps3_approximationA_klmv(x, m2Q2, 1., nf)
                        tmpB = oldad.C2_ps3_approximationB_klmv(x, m2Q2, 1., nf)
                    if tmpA > tmpB:
                        higher = tmpA
                        lower = tmpB
                    else:
                        higher = tmpB
                        lower = tmpA
                    tmp = app.MuIndependentTermsBand(x, m2Q2, nf)
                    np.testing.assert_allclose(higher, tmp.GetHigher(), rtol=1e-7)
                    np.testing.assert_allclose(lower, tmp.GetLower(), rtol=1e-7)

def test_klmv_paper_as3():
    for channel in ['g', 'q']:
        app = ad.ApproximateCoefficientFunctionKLMV(3, '2', channel, "klmv")
        for xi in np.geomspace(1e-2, 1e2, 10):
            m2Q2 = 1/xi
            for x in np.geomspace(1e-5, 1, 10):
                for nf in [4, 5, 6]:
                    if channel == 'g':
                        tmpA = oldad.C2_g3_approximationA_klmv(x, m2Q2, 1., nf, 0)
                        tmpB = oldad.C2_g3_approximationB_klmv_paper(x, m2Q2, 1., nf, 0)
                    if channel == 'q':
                        tmpA = oldad.C2_ps3_approximationA_klmv_paper(x, m2Q2, 1., nf)
                        tmpB = oldad.C2_ps3_approximationB_klmv_paper(x, m2Q2, 1., nf)
                    if tmpA > tmpB:
                        higher = tmpA
                        lower = tmpB
                    else:
                        higher = tmpB
                        lower = tmpA
                    tmp = app.MuIndependentTermsBand(x, m2Q2, nf)
                    np.testing.assert_allclose(higher, tmp.GetHigher(), rtol=1e-7)
                    np.testing.assert_allclose(lower, tmp.GetLower(), rtol=1e-7)

def test_klmv_lowxi_as3():
    app = ad.ApproximateCoefficientFunctionKLMV(3, '2', 'g', "klmv", True)
    for xi in np.geomspace(1e-2, 1e2, 10):
        m2Q2 = 1/xi
        for x in np.geomspace(1e-5, 1, 10):
            for nf in [4, 5, 6]:
                tmpA = oldad.C2_g3_approximationA_klmv(x, m2Q2, 1., nf, 0)
                tmpB = oldad.C2_g3_approximationBlowxi_klmv(x, m2Q2, 1., nf, 0)
                if tmpA > tmpB:
                    higher = tmpA
                    lower = tmpB
                else:
                    higher = tmpB
                    lower = tmpA
                tmp = app.MuIndependentTermsBand(x, m2Q2, nf)
                np.testing.assert_allclose(higher, tmp.GetHigher(), rtol=1e-7)
                np.testing.assert_allclose(lower, tmp.GetLower(), rtol=1e-7)



import adani as ad
import oldadani as old
import numpy as np

CA = 3
CF = 4./3

def test_get_methods():
    for entry1 in ["q", "g"]:
        for entry2 in ["q", "g"]:
            sf = ad.SplittingFunction(0, entry1, entry2)
            assert sf.GetOrder() == 0
            assert sf.GetEntry1() == entry1
            assert sf.GetEntry2() == entry2

def test_Pgq0():

    sf = ad.SplittingFunction(0, 'g', 'q')

    for x in np.geomspace(1e-5, 1., 10, endpoint=True):
        for nf in range(1, 6 + 1):
            np.testing.assert_allclose(old.Pgq0(x), sf.Regular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.Singular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.SingularIntegrated(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.Local(nf), rtol = 1e-7)

def test_Pqg0():

    sf = ad.SplittingFunction(0, 'q', 'g')

    for x in np.geomspace(1e-5, 1., 10, endpoint=True):
        for nf in range(1, 6 + 1):
            np.testing.assert_allclose(old.Pqg0(x, nf), sf.Regular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.Singular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.SingularIntegrated(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.Local(nf), rtol = 1e-7)

def test_Pgg0():

    sf = ad.SplittingFunction(0, 'g', 'g')

    for x in np.geomspace(1e-5, 1., 10, endpoint=False):
        for nf in range(1, 6 + 1):
            np.testing.assert_allclose(old.Pgg0reg(x), sf.Regular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(old.Pgg0sing(x), sf.Singular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(old.Pgg0sing_integrated(x), sf.SingularIntegrated(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(old.Pgg0loc(nf), sf.Local(nf), rtol = 1e-7)

def test_Pqq0():

    sf = ad.SplittingFunction(0, 'q', 'q')

    for x in np.geomspace(1e-5, 1., 10, endpoint=False):
        for nf in range(1, 6 + 1):
            np.testing.assert_allclose(old.Pqq0reg(x), sf.Regular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(old.Pqq0sing(x), sf.Singular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(old.Pqq0sing_integrated(x), sf.SingularIntegrated(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(old.Pqq0loc(), sf.Local(nf), rtol = 1e-7)


def test_Pgq1():

    sf = ad.SplittingFunction(1, 'g', 'q')

    for x in np.geomspace(1e-5, 1., 10, endpoint=False):
        for nf in range(1, 6 + 1):
            np.testing.assert_allclose(old.Pgq1(x, nf), sf.Regular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.Singular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.SingularIntegrated(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.Local(nf), rtol = 1e-7)

def test_Pgg1():

    sf = ad.SplittingFunction(1, 'g', 'g')

    for x in np.geomspace(1e-5, 1., 10, endpoint=False):
        for nf in range(1, 6 + 1):
            np.testing.assert_allclose(old.Pgg1reg(x, nf), sf.Regular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(old.Pgg1sing(x, nf), sf.Singular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(old.Pgg1sing_integrated(x, nf), sf.SingularIntegrated(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(old.Pgg1loc(nf), sf.Local(nf), rtol = 1e-7)


def Pgg0_x_Pgq0(x, nf):
    return ((-4. * CF * nf * (2. + (-2. + x) * x)
            + 2. * CA * CF
                  * (-40. + x * (26. + x * (17. + 8. * x))
                     + 12. * (2. + (-2. + x) * x) * np.log(1. - x)
                     - 24. * (1. + x + x * x) * np.log(x)))
           / 3. / x)

def test_Pgg0_x_Pgq0():

    sf = ad.ConvolutedSplittingFunctions(0, 'g', 'g', 0, 'g', 'q')

    for x in np.geomspace(1e-5, 1., 10, endpoint=False):
        for nf in range(1, 6 + 1):
            np.testing.assert_allclose(Pgg0_x_Pgq0(x, nf), sf.Regular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.Singular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.SingularIntegrated(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.Local(nf), rtol = 1e-7)

def Pqq0_x_Pgq0(x):
    return ((2. * CF * CF
            * (4. * (2. + (-2. + x) * x) * np.log(1. - x)
               - x * (-4. + x + 2. * (-2. + x) * np.log(x))))
           / x)

def test_Pqq0_x_Pgq0():

    sf = ad.ConvolutedSplittingFunctions(0, 'q', 'q', 0, 'g', 'q')

    for x in np.geomspace(1e-5, 1., 10, endpoint=False):
        for nf in range(1, 6 + 1):
            np.testing.assert_allclose(Pqq0_x_Pgq0(x), sf.Regular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.Singular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.SingularIntegrated(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.Local(nf), rtol = 1e-7)

def Pgq0_x_Pqg0(x, nf):
    return (4. * CF * nf
           * (1. + 4. / 3 / x - x - 4. * x * x / 3 + 2. * (1 + x) * np.log(x)))

def test_Pgq0_x_Pqg0():

    sf = ad.ConvolutedSplittingFunctions(0, 'g', 'q', 0, 'q', 'g')

    for x in np.geomspace(1e-5, 1., 10, endpoint=False):
        for nf in range(1, 6 + 1):
            np.testing.assert_allclose(Pgq0_x_Pqg0(x, nf), sf.Regular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.Singular(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.SingularIntegrated(x, nf), rtol = 1e-7)
            np.testing.assert_allclose(0., sf.Local(nf), rtol = 1e-7)



import adani as ad
import oldadani as old
import numpy as np

def test_as1():
    nf = 1
    for kind in ['2', 'L']:
        cf = ad.ExactCoefficientFunction(1, kind, 'g')
        for xi in np.geomspace(1e-2, 1e4, 10, endpoint=True):
            xmax = 1/(1 + 4/xi)
            for x in np.geomspace(1e-5, xmax, 100, endpoint=False):
                res1 = cf.fx(x, 1./xi, 1./xi, nf)
                res2 = old.C2_g1(x, 1./xi) if kind == '2' else old.CL_g1(x, 1./xi)
                np.testing.assert_allclose(res1, res2, rtol = 1e-7)

def test_as2_muindep():
    nf = 1
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            cf = ad.ExactCoefficientFunction(2, kind, channel, 1e-3, 1e-3, 1000)
            for xi in np.geomspace(1e-2, 1e4, 10, endpoint=True):
                for eta in np.geomspace(1e-5, 1e5, 10, endpoint=False):
                    x = 1. / (1. + 4. * 1/xi * (eta + 1))
                    res1 = cf.MuIndependentTerms(x, 1./xi, nf)
                    if kind == '2' and channel == 'g':
                        res2 = old.C2_g20(x, 1/xi)
                    if kind == 'L' and channel == 'g':
                        res2 = old.CL_g20(x, 1/xi)
                    if kind == '2' and channel == 'q':
                        res2 = old.C2_ps20(x, 1/xi)
                    if kind == 'L' and channel == 'q':
                        res2 = old.CL_ps20(x, 1/xi)

                    np.testing.assert_allclose(res1, res2, rtol = 1e-7)

def test_as2_order_mudep():
    nf = 1
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            cf = ad.ExactCoefficientFunction(2, kind, channel, 1e-3, 1e-3, 1000)
            for xi in np.geomspace(1e-2, 1e4, 10, endpoint=True):
                for ximu in np.geomspace(1e-2, 1e4, 10, endpoint=True):
                    xmax = 1/(1 + 4/xi)
                    for x in np.geomspace(1e-5, xmax, 10, endpoint=False):
                        res1 = cf.MuDependentTerms(x, 1./xi, 1./ximu, nf)
                        if kind == '2' and channel == 'g':
                            res2 = old.C2_g21(x, 1/xi)
                        elif kind == 'L' and channel == 'g':
                            res2 = old.CL_g21(x, 1/xi)
                        elif kind == '2' and channel == 'q':
                            res2 = old.C2_ps21(x, 1/xi)
                        else:
                            res2 = old.CL_ps21(x, 1/xi)
                        np.testing.assert_allclose(res1, res2 * np.log(ximu), rtol = 1e-7)

# def test_as3_order_mudep():
#     for kind in ['2', 'L']:
#         for channel in ['g', 'q']:
#             cf = ad.ExactCoefficientFunction(3, kind, channel, 1e-3, 1e-3, 1000, False)
#             for xi in np.geomspace(1e-2, 1e4, 5, endpoint=True):
#                 for ximu in np.geomspace(1e-2, 1e4, 5, endpoint=True):
#                     xmax = 1/(1 + 4/xi)
#                     for x in np.geomspace(1e-5, xmax, 5, endpoint=False):
#                         for nf in range(4, 6 + 1):
#                             res1 = cf.MuDependentTerms(x, 1./xi, 1./ximu, nf)
#                             if kind == '2' and channel == 'g':
#                                 res2 = old.C2_g31(x, 1/xi, nf) * np.log(ximu) + old.C2_g32(x, 1/xi, nf) * np.log(ximu)**2
#                             elif kind == 'L' and channel == 'g':
#                                 res2 = old.CL_g31(x, 1/xi, nf) * np.log(ximu) + old.CL_g32(x, 1/xi, nf) * np.log(ximu)**2
#                             elif kind == '2' and channel == 'q':
#                                 res2 = old.C2_ps31(x, 1/xi, nf) * np.log(ximu) + old.C2_ps32(x, 1/xi, nf) * np.log(ximu)**2
#                             else:
#                                 res2 = old.CL_ps31(x, 1/xi, nf) * np.log(ximu) + old.CL_ps32(x, 1/xi, nf) * np.log(ximu)**2
#                             np.testing.assert_allclose(res1, res2, rtol = 1e-7)



import adani as ad
import numpy as np

nf=4

def test_mc_integrals():
    for kind in ['2', 'L']:
        massive_mc = ad.ExactCoefficientFunction(3, kind, 'g', 1e-3, 1e-3, 1000, "monte_carlo", 1000000)
        massive_nomc = ad.ExactCoefficientFunction(3, kind, 'g', 1e-3, 1e-3, 1000, "double_numerical", 25000)
        massive_analytical = ad.ExactCoefficientFunction(3, kind, 'g', 1e-3, 1e-3, 1000, "analytical")
        for xi in np.geomspace(1e-2, 1e4, 4, endpoint=True):
            m2Q2 = 1/xi
            xmax = 1. / (1. + 4 * m2Q2)
            for x in np.geomspace(1e-5, xmax, 5, endpoint=False):
                    for nf in range(1, 6 + 1):
                        res1 = massive_mc.MuDependentTerms(x, m2Q2, m2Q2, nf)
                        res2 = massive_nomc.MuDependentTerms(x, m2Q2, m2Q2, nf)
                        res3 = massive_analytical.MuDependentTerms(x, m2Q2, m2Q2, nf)
                        np.testing.assert_allclose(res1, res2, rtol=5e-3)
                        np.testing.assert_allclose(res2, res3, 1e-3)



import sys, os
import versioneer

try:
    from skbuild import setup
except ImportError:
    print(
        "Please update pip, you need pip 10 or greater,\n"
        " or you need to install the PEP 518 requirements in pyproject.toml yourself",
        file=sys.stderr,
    )
    raise

from setuptools import find_packages

# load long description from README
this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="adani",
    version=versioneer.get_version(),
    description="Code computing approximate DIS N3LO coefficients",
    author="Niccolò Laurenti",
    license="AGPLv3",
    packages=find_packages(where="inc"),
    package_dir={"": "inc"},
    cmake_install_dir="inc/adani",
    cmake_args=['-DPYTHON_ONLY:BOOL=ON'],
    python_requires=">=3.8",
    long_description=long_description,
    long_description_content_type="text/markdown",
)



import adani as ad
import oldadani as old
import numpy as np

def test_as1():
    for kind in ['2', 'L']:
        ml = ad.MasslessCoefficientFunction(1, kind, 'g')
        for x in np.geomspace(1e-5, 1., 100, endpoint=False):
            for nf in range(6 + 1):
                res1 = ml.MuIndependentTerms(x, nf)
                res2 = old.C2_g1_massless(x, nf) if kind == '2' else old.CL_g1_massless(x, nf)
                np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_as2():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            ml = ad.MasslessCoefficientFunction(2, kind, channel)
            for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                for nf in range(6 + 1):
                    res1 = ml.MuIndependentTerms(x, nf)
                    if (kind == '2' and channel == 'g'):
                        res2 = old.C2_g2_massless(x, nf)
                    if (kind == '2' and channel == 'q'):
                        res2 = old.C2_ps2_massless(x, nf)
                    if (kind == 'L' and channel == 'g'):
                        res2 = old.CL_g2_massless(x, nf)
                    if (kind == 'L' and channel == 'q'):
                        res2 = old.CL_ps2_massless(x, nf)
                    np.testing.assert_allclose(res1, res2, rtol=1e-7)


def test_as3():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            ml = ad.MasslessCoefficientFunction(3, kind, channel)
            for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                for nf in range(6 + 1):
                    res1 = ml.MuIndependentTerms(x, nf)
                    if (kind == '2' and channel == 'g'):
                        res2 = old.C2_g3_massless(x, nf)
                    if (kind == '2' and channel == 'q'):
                        res2 = old.C2_ps3_massless(x, nf)
                    if (kind == 'L' and channel == 'g'):
                        res2 = old.CL_g3_massless(x, nf)
                    if (kind == 'L' and channel == 'q'):
                        res2 = old.CL_ps3_massless(x, nf)
                    np.testing.assert_allclose(res1, res2, rtol=1e-7)
